#include <iostream.h>
#include <qstring.h>
#include "ircLine.moc"
#include "ircLine.h"

IrcLine::IrcLine(QWidget* parent, const char* name, int buff,  bool clean) : 
  QLineEdit(parent, name)
{
#ifdef EDEBUG
  cout << "Konstruktor IrcLine..."<<endl;
#endif
   setValidator(0L);
   autoClean = clean;
   history   = new RingList<char>(buff);
   history->add(qstrdup(""));
   fontSize  = 2;
   fontName  = "times";
   textColor = QColor(black);
   QFont f(fontName, 10+fontSize);
   setFont(f);
   connect(this, SIGNAL(returnPressed()), SLOT(slotTextReturn()));
}

IrcLine::~IrcLine()
{
#ifdef EDEBUG
  cout << "Destruktor IrcLine" << endl;
#endif
  delete history;
}

void IrcLine::mousePressEvent(QMouseEvent* e)
{
  QLineEdit::mousePressEvent(e);
  if (e->button()==RightButton){
    QPoint pos=mapToGlobal(e->pos());
    emit signMouseRightPress(pos);
  }
  setFocus();
}

void IrcLine::slotTextReturn()
{
   QString t = text();
   if (t.isNull() || t.isEmpty())
      return;
   else{
      history->addLast(qstrdup(t));
      emit signReturn(t);
      history->add(qstrdup(""));
      if (autoClean)
	 setText("");
   }
}


void IrcLine::keyPressEvent(QKeyEvent* e)
{
  QString txt = text();
  int k = e->key();

#if(QT_VERSION>=140)

  if (txt.isEmpty()){
     txt="";
     setCursorPosition(0);
  }
  int pos=cursorPosition();

  if (e->state() & ControlButton){
     switch (k) {
     case Key_B:
       txt.insert(pos, 0x02);
       setText(txt);
       setCursorPosition(pos+1);
       return;
       break;
     case Key_U:
       txt.insert(pos, 0x1F);
       setText(txt);
       setCursorPosition(pos+1);
       return;
       break;
     case Key_I:
       txt.insert(pos, 0x16);
       setText(txt);
       setCursorPosition(pos+1);
       return;
       break;
     case Key_K: 
       txt.insert(pos, 0x03);
       setText(txt);
       setCursorPosition(pos+1);
       return;
       break;
     case Key_O:
       txt.insert(pos, 0x0F);
       setText(txt);
       setCursorPosition(pos+1);
       return;
       break;
     }
     return;
  }

  if (k==Key_Down){
      QString s=history->forward();
      setText(s);
      return;
  }
  else if (k==Key_Up){
      setText(history->back());
      return;
  }
  else if (k==Key_Left || k==Key_Right){
      QLineEdit::keyPressEvent(e);
      return;
  }

  int a=e->ascii();
  if ( a==13){
     QLineEdit::keyPressEvent(e);
     return;
  }

  if (e->key() == Key_Delete){
    if (pos < txt.length() && pos>=0){
       txt.remove(pos, 1);
       setText(txt);
       setCursorPosition(pos);
    }
    return;
  }
  if (e->key() == Key_Backspace){
     if (pos<1)
        return;
     txt.remove(pos-1, 1);
     setText(txt);
     setCursorPosition(pos-1);
     return;
  }

  if (a>=32){
     txt.insert(pos, a);
     setText(txt);
     setCursorPosition(pos+1);
  }

#else

  if (e->state() & ControlButton){
     QString txt = text();
     switch (k) {
     case Key_B:
       txt += 0x02;
       setText(txt);
       return;
       break;
     case Key_U:
       txt += 0x1F;
       setText(txt);
       return;
       break;
     case Key_I:
       txt += 0x16;
       setText(txt);
       return;
       break;
     case Key_K:
       txt += 0x03;
       setText(txt);
       return;
       break;
     case Key_O:
       txt += 0x0F;
       setText(txt);
       return;
       break;
     }
  }

  if (k==Key_Down){
      QString s=history->forward();
      setText(s);
      return;
  }
  else if (k==Key_Up){
      setText(history->back());
      return;
  }
  QLineEdit::keyPressEvent(e);

#endif

  setFocus();
}


