#include <iostream.h>
#include <qtooltip.h> 
#include <qcolor.h>
#include "ircClient.h"
#include "outputPage.h"
#include "ircMsgPage.h"
#include "channelPage.h"
#include "privMsgPage.h"
#include "dccChatPage.h"
#include "dccFilePage.h"
#include "colorOpt.h"

#include "ircMDI.h"
#include "ircMDI.moc"

WindowButton::WindowButton(MDIWindow* w, const char* txt, QWidget* p, const char* name)
  : QPushButton(p, name)
{
  label = new QLabel(this);
  label->setAlignment(AlignLeft|AlignVCenter);
  label->setMargin(10);
  label->setText(txt);
  label->setAutoResize(false);
  label->installEventFilter(this);
  label->setBackgroundColor(lightGray);

  setToggleButton(true);
  mdiWnd = w;
  connect(this, SIGNAL(toggled    (bool)),
	  this, SLOT  (slotToggled(bool)));
  connect(w,    SIGNAL(captionChanged(MDIWindow*, const char*)),
	  this, SLOT  (slotChangeText(MDIWindow*, const char*)));
  connect(w,    SIGNAL(minimized(MDIWindow*)),
	  this, SLOT  (setOff()));
}

void WindowButton::setOff()
{
  if (isOn()){
     label->setBackgroundColor(lightGray);
     setOn(false);
  }
}

void WindowButton::slotToggled(bool f)
{
  emit toggled(mdiWnd, f);
  if (f==true){
     label->setBackgroundColor(darkGray);
     mdiWnd->slotSelect();
  }
  else{
     label->setBackgroundColor(lightGray);
     mdiWnd->hide();
  }
}

void WindowButton::slotChangeText(MDIWindow*, const char* txt)
{
  QToolTip::remove(this);
  QToolTip::add(this, txt);
  label->setText(txt);
}

bool WindowButton::eventFilter ( QObject *, QEvent *e )
{
    if (e->type() == Event_MouseButtonPress){
        mousePressEvent( (QMouseEvent *)e );
	return true;
    }
   
    return false;
}


void WindowButton::mousePressEvent(QMouseEvent*)
{
  bool f;
  if (isOn())
     f=false;
  else
     f=true;
  setOn(f);
  emit QButton::toggled(f);
}

void WindowButton::resizeEvent(QResizeEvent* e)
{
  QPushButton::resizeEvent(e);

  label->setGeometry(1,
		     1,
		     width()-2,
		     height()-2);
}


StatusBar::StatusBar(QWidget* parent, const char* name)
  : QWidget(parent, name)
{
  btnList.setAutoDelete(true);
  frame = new QFrame(this);
  frame->setLineWidth(1);
  frame->setFrameStyle(QFrame::Panel|QFrame::Raised);
}

void StatusBar::resizeEvent(QResizeEvent*)
{
  frame->setGeometry(0,
		     0,
		     width(),
		     height());
  updateRect();
}

void StatusBar::updateRect()
{
  int w=width();
  int n=btnList.count();
  int dw;
  if (MAX_WIDTH*n < w)
     dw=MAX_WIDTH;
  else
     dw=w/n;
  for (int i=0;i<n;i++){
     WindowButton* wb=btnList.at(i);
     wb->setGeometry(i*dw,
		    0,
		    dw,
		    22);
  }
}

void StatusBar::addButton(MDIWindow* w)
{
  WindowButton* btn=new WindowButton(w, w->name(), frame);
  btnList.append(btn);
  connect(btn,  SIGNAL(toggled    (MDIWindow*, bool)),
	  this, SLOT  (slotToggled(MDIWindow*, bool)));
  connect(w,    SIGNAL(selected   (MDIWindow*)),
	  this, SLOT  (setOn      (MDIWindow*)));
  setOn(w);
  updateRect();
  btn->show();
}

void StatusBar::delButton(MDIWindow* wnd)
{
  WindowButton* btn;
  for (btn=btnList.first();btn!=0L;btn=btnList.next()){
      MDIWindow* w=btn->mdiWnd;
      if (w==wnd){
	 btnList.remove();
	 updateRect();
	 return;
      }
  }
}

void StatusBar::slotToggled(MDIWindow* wnd, bool f)
{
  if (f==true){
    setOn(wnd);
    emit selected(wnd);
  }
}

void StatusBar::setOn(MDIWindow* wnd)
{
  WindowButton* btn;
  for (btn=btnList.first();btn!=0L;btn=btnList.next()){
      MDIWindow* w=btn->mdiWnd;
      if (w==wnd){
	 if (!btn->isOn())
	    btn->setOn(true);
      }
      else if (btn->isOn())
	 btn->setOn(false);
  }
}


IrcMDI::IrcMDI(IrcClient* Client, QWidget* p, const char* name)
  : QFrame(p, name)
{
#ifdef EDEBUG
  cout << "Konstruktor IrcMDI" << endl;
#endif
  client = Client;

  QVBoxLayout* vLayout = new QVBoxLayout(this, 0);
  mdi = new MDIManager(this, name,
		       "pindown.xpm",
		       "pinup.xpm",
		       "minimize.xpm",
		       "maximize.xpm",
		       "close.xpm");
  statusBar = new StatusBar(this);

  connect (mdi,      SIGNAL(windowAdded(MDIWindow*)),
	   statusBar,SLOT  (addButton  (MDIWindow*)));
  connect (mdi,      SIGNAL(windowRemoved(MDIWindow*)),
	   statusBar,SLOT  (delButton    (MDIWindow*)));

  connect((ColorOpt*)client->optPrefDlg->getPage("Color"),
	  SIGNAL(colorsChanged(const QColor&, int)),
	  mdi,
	  SLOT  (slotSetColor (const QColor&, int))
	  ); 
  connect((ColorOpt*)client->optPrefDlg->getPage("Color"),
	  SIGNAL(bgChanged(const char*)),
	  mdi,
	  SLOT  (slotSetBgImage(const char*))
	  );  


  vLayout->addWidget(mdi, 10);
  vLayout->addWidget(statusBar);
  statusBar->setFixedHeight(22);
  vLayout->activate();

  // Status
  IrcMsgPage* imp=new IrcMsgPage(client, this);
  MDIWindow* mdiWnd=mdi->addWindow(imp, true, MDI_CLOSED|MDI_DETACHED);
  imp->setMDIWindow(mdiWnd);
  imp->setCaption("Status");


  // From ircTab
  connect(client,
	  SIGNAL(signAddOutputPage(const char*, int, bool)),
          this,
          SLOT  (slotAddOutputPage(const char*, int, bool)));
  connect(client,
          SIGNAL(signDelOutputPage(const char*)),
	  this,
          SLOT  (slotDelOutputPage(const char*)));
  connect(client,
          SIGNAL(signWritePage(const char*, int, const char*, bool)),
          this,
          SLOT  (slotWritePage(const char*, int, const char*, bool)));

  // From channelView
  connect(client, 
          SIGNAL(signAddChannel(const char*, const char*)),
          this,
          SLOT  (slotAddChannel(const char*, const char*)));
  connect(client, 
          SIGNAL(signDelChannel(IrcChannel*)),
          this,
          SLOT  (slotDelChannel(IrcChannel*)));

  // From privMsgView
  connect(client, 
          SIGNAL(signAddPrivMsg(const char*)),
          this,
          SLOT  (slotAddPrivMsg(const char*)));
  connect(client, 
          SIGNAL(signDelPrivMsg(IrcPrivMsg*)),
          this,
          SLOT  (slotDelPrivMsg(IrcPrivMsg*)));

  // From dccChatTab
  connect(client, 
          SIGNAL(signAddDCCChat(const char*)),
          this,
          SLOT  (slotAddDCCChat(const char*)));
  connect(client, 
          SIGNAL(signDelDCCChat(DCCChat*)),
          this,
          SLOT  (slotDelDCCChat(DCCChat*)));

  // From dccFileTab
  connect(client, 
          SIGNAL(signAddDCCFile(const char*)),
          this,
          SLOT  (slotAddDCCFile(const char*)));
  connect(client, 
          SIGNAL(signDelDCCFile(DCCFile*)),
          this,
          SLOT  (slotDelDCCFile(DCCFile*)));

  cpList.setAutoDelete (false);
  pmpList.setAutoDelete(false);
  wpList.setAutoDelete (false);
  dcpList.setAutoDelete(false);
  dfpList.setAutoDelete(false);

  // Background for MDI
  QString entry=ircapp->readEntry("BackgroundImage", "");
  if (!entry.isEmpty()){
     mdi->slotSetBgImage(entry);
  }
  else{
     KConfig *config = ircapp->getConfig();
     config->setGroup( "kEirc Option" );
     QColor bgColor = config->readColorEntry( "BgMDIColor",   &lightGray );
     mdi->slotSetColor(bgColor, TYPE_BGMDI);
  }

#ifdef EDEBUG
  cout << "Konstruktor IrcMDI...Ok" << endl;
#endif
}

void IrcMDI::slotAddOutputPage(const char* name, int flag, bool detach)
{
#ifdef EDEBUG
  cout << "IrcMDI::slotAddOutputPage:"<<name<<":"<<flag<<endl;
#endif
  WindowPage* wp;
  for (wp=wpList.first();wp!=0L;wp=wpList.next()){
      if (stricmp(wp->name(), name))
	 continue;
      return;
  }

  MDIWindow* mdiWnd;
  wp=new WindowPage(client, this, name);
  mdiWnd=mdi->addWindow(wp, true, MDI_DETACHED|MDI_CLOSED);
  wp->setMDIWindow(mdiWnd);
  wp->setCaption(QString("Window Page ")+name);

  wpList.append(wp);

  connect(wp->ircLine(), 
	  SIGNAL(signReturn(const char*)), 
          client, 
          SLOT  (slotInputParse(const char*)));
}

void IrcMDI::slotDelOutputPage(const char* name)
{
#ifdef EDEBUG
  cout << "IrcMDI::slotDelOutputPage:"<<name<<endl;
#endif
  WindowPage* cp;
  for (cp=wpList.first();cp!=0L;cp=wpList.next()){
      if (stricmp(cp->name(), name))
	 continue;
      cp->mdiWnd->slotClose();
      wpList.remove();
      return;
  }
}

void IrcMDI::slotWritePage(const char* name, int Type, const char* Msg, bool Parsen)
{
  if (name==0L)
     return;
  WindowPage* wp;
  for (wp=wpList.first();wp!=0L;wp=wpList.next()){
      if (stricmp(wp->name(), name))
	 continue;
      wp->ircHtml()->slotWrite(Type, Msg, Parsen);
      return;
  }
}

void IrcMDI::slotAddChannel(const char* Channel, const char* Key)
{
#ifdef EDEBUG
  cout << "IrcMDI::slotAddChannel:"<<Channel<<endl;
#endif
  ChannelPage* cp=new ChannelPage(client, this, Channel);
  MDIWindow* mdiWnd=mdi->addWindow(cp, true, MDI_DETACHED);
  cp->setMDIWindow(mdiWnd);
  cp->setCaption(Channel);

  cpList.append(cp);
}

void IrcMDI::slotDelChannel(IrcChannel* ic)
{
#ifdef EDEBUG
  cout << "IrcMDI::slotDelChannel:"<<endl;
#endif
  ChannelPage* cp;
  for (cp=cpList.first();cp!=0L;cp=cpList.next()){
      if (cp->getClient()!=ic)
	 continue;
      cp->mdiWnd->slotClose();
      cpList.remove();
      return;
  }
}


void IrcMDI::slotAddPrivMsg(const char* Name)
{
  mdi->removeWindow(Name);
  PrivMsgPage* pp;
  
  pp=new PrivMsgPage(client, this, Name);
  MDIWindow* mdiWnd=mdi->addWindow(pp, true, MDI_DETACHED);
  pp->setMDIWindow(mdiWnd);
  pp->setCaption(Name);

  pmpList.append(pp);

  connect(pp->getClient(), 
          SIGNAL(signChangeNick(const char*)),
          pp,
          SLOT  (setCaption    (const char*)));
}


void IrcMDI::slotDelPrivMsg(IrcPrivMsg* ic)
{
#ifdef EDEBUG
  cout << "IrcMDI::slotDelPrivMsg:"<<endl;
#endif
  PrivMsgPage* cp;
  for (cp=pmpList.first();cp!=0L;cp=pmpList.next()){
      if (cp->getClient()!=ic)
	 continue;
      cp->mdiWnd->slotClose();
      pmpList.remove();
      return;
  }
}

void IrcMDI::slotAddDCCChat(const char* Name)
{
  DCCChatPage* dcp=new DCCChatPage(client, this, Name);
  MDIWindow* mdiWnd=mdi->addWindow(dcp, true, MDI_DETACHED);
  dcp->setMDIWindow(mdiWnd);
  dcp->setCaption(QString("DCC Chat ")+Name);

  dcpList.append(dcp);
}

void IrcMDI::slotDelDCCChat(DCCChat* ic)
{
#ifdef EDEBUG
  cout << "IrcMDI::slotDelDCCChat:"<<endl;
#endif
  DCCChatPage* cp;
  for (cp=dcpList.first();cp!=0L;cp=dcpList.next()){
      if (cp->getClient()!=ic)
	 continue;
      cp->mdiWnd->slotClose();
      dcpList.remove();
      return;
  }
}


void IrcMDI::slotAddDCCFile(const char* Name)
{
  DCCFilePage* dfp = new DCCFilePage(client, this, Name);
  MDIWindow* mdiWnd=mdi->addWindow(dfp, true, MDI_DETACHED);
  dfp->setMDIWindow(mdiWnd);
  dfp->setCaption(QString("DCC File ")+Name);

  dfpList.append(dfp);
}

void IrcMDI::slotDelDCCFile(DCCFile* ic)
{
#ifdef EDEBUG
  cout << "IrcMDI::slotDelDCCFile:"<<endl;
#endif
  DCCFilePage* cp;
  for (cp=dfpList.first();cp!=0L;cp=dfpList.next()){
      if (cp->getClient()!=ic)
	 continue;
      cp->mdiWnd->slotClose();
      dfpList.remove();
      return;
  }
}




