/********************************************************************************** 
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <kapp.h>
#include <qpalette.h>
#include <qpainter.h>


#include "cube.h"


// initialize static classelements

int Cube::player1_points=0;
int Cube::player2_points=0;
int Cube::areWorking=0;
int Cube::akt_player=1;
int Cube::cube_counter=0;
bool Cube::gameover=FALSE;
QPalette* Cube::color1;
QPalette* Cube::color2;


// construktor of Cube

Cube::Cube(QWidget *parent,const char *name ,int max_Value)  :QFrame(parent,name)
{
  setFrameStyle(Panel|Raised);
  setLineWidth(2);
  cube_counter++;
  isWorking=FALSE;
  points=1; // value of the cube
  owner=0; // set owner to nobody
  maxValue=max_Value;
  repaint();  // show value

  

   // after flowing over decrease points
  connect(this,SIGNAL(overflow()),SLOT(decrease()));
}

Cube::~Cube()
{
  cube_counter--;
}

// public functions

void Cube::setMaxValue(int wert)
{
  maxValue=wert;
}


void Cube::setColors(QPalette *player1, QPalette *player2)
{
  color1=player1;
  color2=player2;
}


int Cube::getAktPlayer()
{
  return akt_player;
}


void Cube::setAktPlayer(int newPlayer)
{
  akt_player=newPlayer;
}


// setOwner checks, if owner changed

void Cube::setOwner(int newOwner)
{ 
  if(newOwner==1 && owner==2)
    {
      setPalette(*color1); // backgroundcolor to color of player1
      player1_points+=1; // increase points of player 1
      player2_points-=1; // decrease points of player 2
      if(player1_points==cube_counter)
	{
	  gameover=TRUE;
	  emit gameEnded(1);
	}
      
      owner=1; // set new owner
    }

  else if (newOwner==2 && owner==1)
    {
      setPalette(*color2); // backgroundcolor to color of player 2
     
      player2_points+=1; // increase points of player 2
      player1_points-=1; // decrease points of player 1
      if(player2_points== cube_counter)
	{ 
	  gameover=TRUE;
	  emit gameEnded(2);
	}
      owner=2; // set new owner
    }  
  else if(owner==0)
    {
      if(newOwner==1)
	{ 
	  // backgroundcolor to color of player1
	  setPalette(*color1);
	  owner=1; // set new owner
	  player1_points+=1; // increase points of player 1
	   if(player1_points== cube_counter)
	     {  
	       gameover=TRUE;
	       emit gameEnded(1);
	     }
	}
      else if(newOwner==2)
	{// backgroundcolor to color of player 2
	  setPalette(*color2);
	  owner=2; // set new owner
	  player2_points+=1; // increase points of player 2
	   if(player2_points== cube_counter)
	     { 
	       gameover=TRUE;
	       emit gameEnded(2);
	     }
	}
    }

}

void Cube::setValue(int value)
{
  if(value > maxValue || value < 1)
    return;

  points=value;
  repaint();
}

int Cube::getOwner()
{
  return owner;
}


int Cube::value()
{
  return points;
}


void Cube::startOverflow()
{
  isWorking=TRUE;
  areWorking++;
  
  overflowTimer=startTimer(750);

}



// SLOTS

void Cube::increase()
{ 
  // do following only if game not finished
  if(!gameover)
  {
    // necessary, because the Mouseclickevent changed akt_player 
    if(akt_player==1)
      setOwner(2);
    else if(akt_player==2)
      setOwner(1);
    
    points++;
    
    if(points <= 9)  
      repaint();
    
    if(points>maxValue)
      {
	if(!isWorking)
	  {
	    startOverflow();
	  }
      }
  }
}

void Cube::decrease()
{
  points=points-maxValue;

  if(points>maxValue)
    startOverflow();

  if(points<9)
    repaint();
} 


void Cube::reset()
{ 
 
  gameover=FALSE;
  player1_points=0;
  player2_points=0;
  areWorking=0;
  akt_player=1;
  emit playerChanged(1);
  points=1;
  owner=0;
  repaint();
  setPalette(*kapp->palette());
}

void Cube::updateColor()
{
  if(owner==1)
    setPalette(*color1);
  else if(owner==2)
    setPalette(*color2);
}


// EVENTS

void Cube::mouseReleaseEvent(QMouseEvent *e)
{
  // only accept click if it was inside this cube
  if(e->x()< 0 || e->x() > this->width() || e->y() < 0 || e->y() > this->height())
    return;

  if(areWorking==0 && !gameover)
    {
      if(e->button() == LeftButton)
	if(owner==akt_player || owner==0)
	  {
	    if(akt_player==1)
	      akt_player=2;
	    else akt_player=1;
	    emit playerChanged(akt_player);
	    increase();
	  }
    }
}

void Cube::timerEvent(QTimerEvent *e)
{
  if(e->timerId()==overflowTimer)
  {
    killTimers();
    areWorking--;
    isWorking=FALSE;
    if(!gameover)
      {
	emit overflow();
      }
  }
  
  else debug("unexpected timerevent occured"); 

} 

void Cube::drawContents(QPainter *p)
{
  int size=height();
  int circleSize=size/7;
  QBrush brush("black");
  QPen pen("black");
  p->setBrush(brush);
  p->setPen(pen);
  switch(points)
    {
    case 1:
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,circleSize,circleSize);
      break;

    case 3:
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,circleSize,circleSize);
    case 2:
      p->drawEllipse(size/4-circleSize/2,size/4-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/4-circleSize/2,
		     circleSize,circleSize);
      break;
      
    case 5:
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,circleSize,circleSize);
    case 4:
      p->drawEllipse(size/4-circleSize/2,size/4-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,size/4-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,3*size/4-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/4-circleSize/2,
		     circleSize,circleSize);
      break;

    case 8:
      p->drawEllipse(size/2-circleSize/2,2*size/3-circleSize/2,
		     circleSize,circleSize);
    case 7:
      p->drawEllipse(size/2-circleSize/2,size/3-circleSize/2,
		     circleSize,circleSize);
    case 6:
      p->drawEllipse(size/4-circleSize/2,size/6-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      break;
     
   
    case 9:
      p->drawEllipse(size/4-circleSize/2,size/6-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/2-circleSize/2,2*size/7-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/2-circleSize/2,5*size/7-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,
		     circleSize,circleSize);
      break; 

    default:
      debug("cube had value %d",points);
      QString s;
      s.sprintf("%d",points);
      p->drawText(size/2,size/2,s);
      break;
    }

}



#include "cube.moc"










