/********************************************************************************** 
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  


#include <kapp.h>
#include <kmsgbox.h>
#include <qpalette.h>
#include <klocale.h>
#include <kconfig.h>
#include <kconfigbase.h>

#include "cubebox.h"

#define MAX_CUBESIZE 96
#define MIN_CUBESIZE 36
#define DEFAULT_CUBESIZE 60

CubeBox::CubeBox(int dim,int zoom, QPalette *color1, QPalette *color2,
		 QWidget *parent, const char *name) : QWidget(parent,name)
{
  int i,j;

  // size of playfield
  cubeDim=dim;
  // size of cube
  cubeSize=DEFAULT_CUBESIZE/5*zoom;
    
  // create cubes
  cubes=new Cube**[cubeDim];
  for(i=0;i<cubeDim;i++)
    cubes[i]=new Cube*[cubeDim];
  for(i=0;i<cubeDim;i++)
    for(j=0;j<cubeDim;j++)
      {
	cubes[i][j]=new Cube(this);
	}

  // connect cubes

  // connect cornercubes
  connect(cubes[0][0],SIGNAL(overflow()),cubes[1][0],SLOT(increase()));
  connect(cubes[0][0],SIGNAL(overflow()),cubes[0][1],SLOT(increase()));
  connect(cubes[0][cubeDim-1],SIGNAL(overflow())
	  ,cubes[0][cubeDim-2],SLOT(increase()));
  connect(cubes[0][cubeDim-1],SIGNAL(overflow())
	  ,cubes[1][cubeDim-1],SLOT(increase()));
  connect(cubes[cubeDim-1][0],SIGNAL(overflow())
	  ,cubes[cubeDim-2][0],SLOT(increase()));
  connect(cubes[cubeDim-1][0],SIGNAL(overflow())
	  ,cubes[cubeDim-1][1],SLOT(increase()));
  connect(cubes[cubeDim-1][cubeDim-1],SIGNAL(overflow())
	  ,cubes[cubeDim-2][cubeDim-1],SLOT(increase()));
  connect(cubes[cubeDim-1][cubeDim-1],SIGNAL(overflow())
	  ,cubes[cubeDim-1][cubeDim-2],SLOT(increase()));

  // connect edgecubes
  for(i=1;i<cubeDim-1;i++)
    {
      connect(cubes[0][i],SIGNAL(overflow()),cubes[0][i-1],SLOT(increase()));
      connect(cubes[0][i],SIGNAL(overflow()),cubes[0][i+1],SLOT(increase()));
      connect(cubes[0][i],SIGNAL(overflow()),cubes[1][i],SLOT(increase()));
      connect(cubes[i][0],SIGNAL(overflow()),cubes[i-1][0],SLOT(increase()));
      connect(cubes[i][0],SIGNAL(overflow()),cubes[i+1][0],SLOT(increase()));
      connect(cubes[i][0],SIGNAL(overflow()),cubes[i][1],SLOT(increase()));
      connect(cubes[cubeDim-1][i],SIGNAL(overflow())
	      ,cubes[cubeDim-1][i-1],SLOT(increase()));
      connect(cubes[cubeDim-1][i],SIGNAL(overflow())
	      ,cubes[cubeDim-1][i+1],SLOT(increase()));
      connect(cubes[cubeDim-1][i],SIGNAL(overflow())
	      ,cubes[cubeDim-2][i],SLOT(increase()));
      connect(cubes[i][cubeDim-1],SIGNAL(overflow())
	      ,cubes[i-1][cubeDim-1],SLOT(increase()));
      connect(cubes[i][cubeDim-1],SIGNAL(overflow())
	      ,cubes[i+1][cubeDim-1],SLOT(increase()));
      connect(cubes[i][cubeDim-1],SIGNAL(overflow())
	      ,cubes[i][cubeDim-2],SLOT(increase()));
    }

  // connect remaining cubes
  for(i=1;i<cubeDim-1;i++)
    for(j=1;j<cubeDim-1;j++)
      {
	connect(cubes[i][j],SIGNAL(overflow()),cubes[i][j-1],SLOT(increase()));
	connect(cubes[i][j],SIGNAL(overflow()),cubes[i][j+1],SLOT(increase()));
	connect(cubes[i][j],SIGNAL(overflow()),cubes[i-1][j],SLOT(increase()));
	connect(cubes[i][j],SIGNAL(overflow()),cubes[i+1][j],SLOT(increase()));
      }

  // set maxValues 

   for(i=0;i<cubeDim;i++)
    for(j=0;j<cubeDim;j++)
      {
	if(i==0 || i==(cubeDim-1))
	  if(j==0 || j==(cubeDim-1))
	      cubes[i][j]->setMaxValue(2); // maxValue of the cornercubes
	  else cubes[i][j]->setMaxValue(3);  // left and right egde
	else if(j==0 || j==(cubeDim-1))
	  cubes[i][j]->setMaxValue(3);  // upper and lower edge;
	else cubes[i][j]->setMaxValue(4); // remaining
      }

   for(i=0;i<cubeDim;i++)
     for(j=0;j<cubeDim;j++)
     {
       connect(parent, SIGNAL(reset()) , cubes[i][j] , SLOT(reset()) );
       // show, if next player is on turn
       connect(cubes[i][j], SIGNAL(playerChanged(int))
	       ,  SIGNAL(playerChanged(int)));
       // show, if game is finished
       connect(cubes[i][j],SIGNAL(gameEnded(int)),SLOT(showWinner(int)));
       // when colors changed, show new colors
       connect(this,SIGNAL(colorChanged()),cubes[i][j],SLOT(updateColor()));
     }

   // set colors and sizes

   // set new Colors
   Cube::setColors(color1,color2); // colors are static in Cubes
   
   // set new size
   setFixedSize(cubeDim*cubeSize,cubeDim*cubeSize);
}


// Slots

void CubeBox::showWinner(int winner)
{
  QString s;
  s.sprintf(klocale->translate("Winner is Player %d"),winner);
  KMsgBox::message(this,klocale->translate("Winner"),s);
}


void CubeBox::setZoom(int zoom)
{
  // set new cubesize
  cubeSize=DEFAULT_CUBESIZE/5*zoom;
  setFixedSize(cubeDim*cubeSize,cubeDim*cubeSize);
}


int CubeBox::zoomIn()
{
  if(cubeSize < MAX_CUBESIZE)
    {
      cubeSize+=DEFAULT_CUBESIZE/10;
      setFixedSize(cubeDim*cubeSize,cubeDim*cubeSize);
    }

  return cubeSize*5/DEFAULT_CUBESIZE;
}

int CubeBox::zoomOut()
{
  if(cubeSize > MIN_CUBESIZE)
    {
      cubeSize-=DEFAULT_CUBESIZE/10;
      setFixedSize(cubeDim*cubeSize,cubeDim*cubeSize);
    }
  
  return cubeSize*5/DEFAULT_CUBESIZE;


}


bool CubeBox::maxZoom()
{
  if(cubeSize>=MAX_CUBESIZE)
    return TRUE;
  else return FALSE;
}

bool CubeBox::minZoom()
{
  if(cubeSize<=MIN_CUBESIZE)
    return TRUE;
  else return FALSE;
}

int CubeBox::zoom()
{
  return cubeSize*5/DEFAULT_CUBESIZE;
}

int CubeBox::dim()
{
  return cubeDim;
}

void CubeBox::setColors(QPalette *color1,QPalette *color2)
{
  Cube::setColors(color1,color2);
  emit colorChanged();
}



// Events


void CubeBox::resizeEvent(QResizeEvent *)
{
  // rearrange Cubes
  
  for(int i=0;i< cubeDim;i++)
    for(int j=0;j<cubeDim;j++)
      cubes[i][j]->setGeometry(i*cubeSize,j*cubeSize,cubeSize,cubeSize);
 
}


void CubeBox::saveGame(KConfig *config)
{
  // save aktual player
  config->writeEntry("onTurn",Cube::getAktPlayer());
  
  QStrList list;
  list.setAutoDelete(TRUE);
  QString owner, value, key;
  
  for(int row=0; row < cubeDim ; row++)
    for(int column=0; column < cubeDim ; column++)
      {
	key.sprintf("%u,%u",row,column);
	owner.sprintf("%u",cubes[row][column]->getOwner());
	value.sprintf("%u",cubes[row][column]->value());
	list.append((const char *) owner);
	list.append((const char *) value);
	config->writeEntry((const char *)key , list);

	list.clear();
      }

}

void CubeBox::restoreGame(KConfig *config)
{  
  // set aktual player
  int onTurn=config->readNumEntry("onTurn",1);
  Cube::setAktPlayer(onTurn);
  emit playerChanged(onTurn);
  

  QStrList list;
  list.setAutoDelete(TRUE);
  QString owner, value, key;
  
  for(int row=0; row < cubeDim ; row++)
    for(int column=0; column < cubeDim ; column++)
      {
	key.sprintf("%u,%u",row,column);
	config->readListEntry((const char *) key, list);
	owner=list.first();
	value=list.next();
	cubes[row][column]->setOwner(owner.toInt());
	cubes[row][column]->setValue(value.toInt());
	
	list.clear();
      }
}


#include "cubebox.moc"


