/********************************************************************************** 
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <kapp.h>
#include <kmenubar.h>
#include <ktoolbar.h>
#include <kstatusbar.h>
#include <kstdaccel.h>
#include <qkeycode.h>
#include <qaccel.h>
#include <kmsgbox.h>
#include <kiconloader.h>
#include <klocale.h>
#include <qpalette.h>
#include <kconfig.h>
#include <kcolordlg.h>


#include "topwindow.h"
#include "cubebox.h"


#define DEFAULT_ZOOM 5
#define DEFAULT_DIM 8

TopWindow::TopWindow(const char *name) : KTopLevelWidget(name)
{
  int cubeDim;
  int zoom;

  // create menus

  createMenubar();
  createToolbar();
  createStatusbar();
  
  if(!kapp->isRestored())
    {
      KConfig *config=kapp->getConfig();
      if(config != 0)
	{
	  zoom=config->readNumEntry("Zoom",DEFAULT_ZOOM);
	  cubeDim=config->readNumEntry("CubeDim",DEFAULT_DIM);
	  
	  int menuPos = config->readNumEntry("MenubarPos",(int)(KMenuBar::Top));
	  menu->setMenuBarPos((KMenuBar::menuPosition)menuPos);
	  
	  int barPos = config->readNumEntry("ToolbarPos",(int)(KToolBar::Top));
	  toolbar->setBarPos((KToolBar::BarPosition)barPos);
	  
	  bool visible=(bool)(config->readNumEntry("Toolbar",(int)(TRUE)));
	  viewMenu->setItemChecked(ID_VIEW_TOOLBAR,visible);
	  if(visible)
	    enableToolBar(KToolBar::Show);
	  else 
	    enableToolBar(KToolBar::Hide);
	  
	  
	  visible=(bool)(config->readNumEntry("Statusbar",(int)(TRUE)));
	  viewMenu->setItemChecked(ID_VIEW_STATUSBAR,visible);
	  if(visible)
	    enableStatusBar(KStatusBar::Show);
	  else 
	    enableStatusBar(KStatusBar::Hide);
	  
	  
	  color1=new QPalette(config->readColorEntry("Color1",&QColor("darkred")));
	  color2=new QPalette(config->readColorEntry("Color2",&QColor("darkblue")));
	  
	}      
      else 
	{
	  setDefaultValues(cubeDim,zoom);
	  debug("using default values");
	}
      
      createNewCubebox(cubeDim,zoom);
      
      updatePlayfieldMenu(cubeDim);
      updateZoomMenu(cubebox->zoom());
      
      show();
    }
    
}


TopWindow::~TopWindow()
{
  if(cubebox)
      delete cubebox;
  if(statusbar)
      delete statusbar;
  if(toolbar)
      delete toolbar;
  if(menu)
      delete menu;
  if(playfieldMenu)
      delete playfieldMenu;
  
}



void TopWindow::createMenubar()
{
  // create KStdAccel-object
  KStdAccel *stdaccel = new KStdAccel( kapp->getConfig() );
  // don't know why kjumpingCube segmentfaults by quitting when I use:
  //KStdAccel stdaccel( kapp->getConfig() );  

  QPopupMenu *game = new QPopupMenu; 
  game->insertItem( klocale->translate("&New Game"),ID_GAME_NEW);
  game->insertSeparator();
  game->insertItem(klocale->translate("&Open Game"),ID_GAME_OPEN);
  game->insertItem(klocale->translate("&Save Game"),ID_GAME_SAVE);
  game->insertSeparator();
  game->insertItem(klocale->translate("&Quit"),ID_GAME_QUIT);
  
  game->setAccel(stdaccel->openNew(),ID_GAME_NEW);
  game->setAccel(stdaccel->open(),ID_GAME_OPEN);
  game->setAccel(stdaccel->save(),ID_GAME_SAVE);
  game->setAccel(stdaccel->quit(),ID_GAME_QUIT);

  connect(game,SIGNAL(activated(int)),SLOT(menuCallback(int)));

  zoomMenu =new QPopupMenu;
  zoomMenu->setCheckable(TRUE);
  zoomMenu->insertItem("60%",ID_ZOOM_BASE+3);
  zoomMenu->insertItem("80%",ID_ZOOM_BASE+4);
  zoomMenu->insertItem("100%",ID_ZOOM_BASE+5);
  zoomMenu->insertItem("120%",ID_ZOOM_BASE+6);
  zoomMenu->insertItem("140%",ID_ZOOM_BASE+7);
  zoomMenu->insertItem("160%",ID_ZOOM_BASE+8);

  connect(zoomMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));

  viewMenu=new QPopupMenu;
  viewMenu->setCheckable(TRUE);
  viewMenu->insertItem(klocale->translate("Zoom &In"),ID_OPT_ZOOMIN);
  viewMenu->insertItem(klocale->translate("Zoom &Out"),ID_OPT_ZOOMOUT);
  viewMenu->insertItem(klocale->translate("Set Z&oom"),zoomMenu);
  viewMenu->insertSeparator();
  viewMenu->insertItem(klocale->translate("Show &Toolbar"),ID_VIEW_TOOLBAR);
  viewMenu->insertItem(klocale->translate("Show &Statusbar"),ID_VIEW_STATUSBAR);
  
  viewMenu->setAccel(CTRL+Key_Plus,ID_OPT_ZOOMIN);
  viewMenu->setAccel(CTRL+Key_Minus,ID_OPT_ZOOMOUT);

  connect(viewMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));


  playfieldMenu=new QPopupMenu;
  playfieldMenu->setCheckable(TRUE);
  playfieldMenu->insertItem("&5x5",ID_FIELD +5);
  playfieldMenu->insertItem("&6x6",ID_FIELD +6);
  playfieldMenu->insertItem("&7x7",ID_FIELD +7);
  playfieldMenu->insertItem("&8x8",ID_FIELD +8);
  playfieldMenu->insertItem("&9x9",ID_FIELD +9);
  playfieldMenu->insertItem("&10x10",ID_FIELD +10);
  
  connect(playfieldMenu,SIGNAL(activated(int)),SLOT(menuCallback(int)));

  options = new QPopupMenu;
  options->insertItem(klocale->translate("&Playfield"),playfieldMenu);
  options->insertItem(klocale->translate("Color Player &1"),ID_COLOR_BASE+1);
  options->insertItem(klocale->translate("Color Player &2"),ID_COLOR_BASE+2);
  options->insertItem(klocale->translate("&Default Values"),ID_OPT_DEFAULT);
  options->insertSeparator();
  options->insertItem(klocale->translate("&Save Settings"),ID_OPT_SAVE);

  options->setAccel(CTRL+Key_D,ID_OPT_DEFAULT);
  options->setAccel(CTRL+Key_1,ID_COLOR_BASE+1);
  options->setAccel(CTRL+Key_2,ID_COLOR_BASE+2);
 
  connect(options,SIGNAL(activated(int)),SLOT(menuCallback(int)));


  QPopupMenu *help = new QPopupMenu;
  help->insertItem(klocale->translate("&Contents"),ID_HELP_CONTENTS);
  help->insertItem( klocale->translate("&About..."),ID_HELP_ABOUT);
  
  
  connect(help,SIGNAL(activated(int)),SLOT(menuCallback(int)));
 
  help->setAccel(stdaccel->help(),ID_HELP_CONTENTS);

  menu = new KMenuBar( this );
  CHECK_PTR( menu );
  menu->insertItem( klocale->translate("&Game"), game );
  menu->insertItem(klocale->translate("&View"),viewMenu);
  menu->insertItem(klocale->translate("&Options"),options);
  menu->insertSeparator();
  menu->insertItem( klocale->translate("&Help"), help );
  

  connect(menu,SIGNAL(moved(menuPosition)),SLOT(barPositionChanged()));
 
  
  menu->show();
  setMenu(menu);

  // delete stdaccel 

}


void TopWindow::createToolbar()
{  
  KIconLoader *loader = kapp->getIconLoader();

  toolbar = new KToolBar(this);
  CHECK_PTR( toolbar);
  
  //exitbutton
  toolbar->insertButton(loader->loadIcon("exit.xpm"),ID_GAME_QUIT,
			TRUE,klocale->translate("Quit"));
  // restartbutton
  toolbar->insertButton(loader->loadIcon("reload.xpm"),ID_GAME_NEW,
			TRUE,klocale->translate("New Game"));

  toolbar->insertSeparator();

  // zoominbutton
  toolbar->insertButton(loader->loadIcon("viewmag+.xpm"),ID_OPT_ZOOMIN,
			TRUE,klocale->translate("Zoom In"));
  // zoomoutButton
  toolbar->insertButton(loader->loadIcon("viewmag-.xpm"),ID_OPT_ZOOMOUT,
			TRUE,klocale->translate("Zoom Out"));

  toolbar->insertSeparator();

  //helpbutton
  toolbar->insertButton(loader->loadIcon("help.xpm"),ID_HELP_CONTENTS,TRUE,
			klocale->translate("Help"));

    
  connect(toolbar, SIGNAL(clicked(int)), this, SLOT(menuCallback(int)));
  connect(toolbar,SIGNAL(moved(BarPosition)),SLOT(barPositionChanged()));

 
  toolbar->show();
  addToolBar(toolbar);

}



void TopWindow::createStatusbar()
{  
  statusbar=new KStatusBar(this);
  CHECK_PTR( statusbar );
  statusbar->insertItem(klocale->translate("on turn: Player 1   "),ID_STATUS_TURN);
  statusbar->show();
  setStatusBar(statusbar);
 
}
 
void TopWindow::createNewCubebox(int dim, int size)
{
  cubebox=new CubeBox(dim , size ,color1,color2,this,"cubeBox");

  connect(cubebox,SIGNAL(playerChanged(int)),SLOT(changePlayer(int)));

  cubebox->show();
  setView(cubebox);
  updateRects();

}

void TopWindow::setDefaultValues(int & cubeDim, int &zoom)
{
  zoom=DEFAULT_ZOOM;
  cubeDim=DEFAULT_DIM;
  // set playercolors
  if(color1)
    delete color1;
  if(color2)
    delete color2;
  QColor newColor1("darkred");
  QColor newColor2("darkblue");
  color1=new QPalette(newColor1);
  color2=new QPalette(newColor2);
}


void TopWindow::updatePlayfieldMenu(int dim)
{
  for(int i=5;i<=10;i++)
    playfieldMenu->setItemChecked(ID_FIELD +i ,FALSE);
  
  playfieldMenu->setItemChecked(ID_FIELD +dim,TRUE);
}

void TopWindow::updateZoomMenu(int zoom)
{
  for(int i=2;i<=8;i++)
    zoomMenu->setItemChecked(ID_ZOOM_BASE+i,FALSE);

  zoomMenu->setItemChecked(ID_ZOOM_BASE+zoom,TRUE);

  if(cubebox->maxZoom())
    {
      menu->setItemEnabled(ID_OPT_ZOOMIN,FALSE);
      toolbar->setItemEnabled(ID_OPT_ZOOMIN,FALSE);
    }
  else
    {
      menu->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
      toolbar->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
    }
  if(cubebox->minZoom())
    {
      menu->setItemEnabled(ID_OPT_ZOOMOUT,FALSE);
      toolbar->setItemEnabled(ID_OPT_ZOOMOUT,FALSE);
    }
  else
    {
      menu->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
      toolbar->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
    }

}	     
   

void TopWindow::changeColor(int player)
{
  QColor newColor;
  int result=KColorDialog::getColor(newColor);
  if(result)
    {
      if(player==1)
	{
	  delete color1;
	  color1=new QPalette(newColor);
	}    
      else if(player==2) 
	{
	  delete color2;
	  color2=new QPalette(newColor);
	}
      cubebox->setColors(color1,color2);
    }
} 


void TopWindow::saveSettings()
{
  KConfig *config=kapp->getConfig();
  bool status=viewMenu->isItemChecked(ID_VIEW_TOOLBAR);
  config->writeEntry("Toolbar",(int)(status));
  status=viewMenu->isItemChecked(ID_VIEW_STATUSBAR);
  config->writeEntry("Statusbar",(int)(status));
  config->writeEntry("Zoom",cubebox->zoom());
  config->writeEntry("CubeDim",cubebox->dim());
  config->writeEntry("Color1",color1->normal().background());
  config->writeEntry("Color2",color2->normal().background());

  config->sync();
}

 
// SLOT for handling menubar- and toolbarevents

void TopWindow::menuCallback(int item)
{
  // what is pressed

  switch(item)
    {
    case ID_GAME_QUIT:
      {
	kapp->quit();
	break;
      }
    case ID_GAME_NEW:
      emit reset();
      break;
      
    case ID_GAME_OPEN:
      {
	KConfig *config=kapp->getConfig();
	KConfigGroupSaver groupSaver(config,QString("SavedGame"));
	int newDim=config->readNumEntry("CubeDim");
	if(newDim==0) // if no game was saved before
	  {
	    QString s(klocale->translate("There is no saved Game !"));
	    KMsgBox::message( 0, klocale->translate("Information"), 
			      (const char *)s,
			      KMsgBox::INFORMATION, klocale->translate("OK") );
	  }
	else
	  {
	     if(cubebox->dim() != newDim)
	       {
		 int zoom=cubebox->zoom();
		 updatePlayfieldMenu(newDim);
		 // create new cubebox
		 delete cubebox;
		 createNewCubebox(newDim,zoom);
	       }
	    cubebox->restoreGame(config);
	  }
	break;
      }
     

    case ID_GAME_SAVE:
      {
	KConfig *config=kapp->getConfig();
	KConfigGroupSaver groupSaver(config,QString("SavedGame"));
	config->writeEntry("CubeDim",cubebox->dim());
	cubebox->saveGame(kapp->getConfig());
	QString s(klocale->translate("Game saved !"));
	KMsgBox::message( 0, klocale->translate("Information"), 
			  (const char *)s,
			  KMsgBox::INFORMATION, klocale->translate("OK") );
	break;
      }
    case ID_VIEW_TOOLBAR:
      {
	enableToolBar(); // toggles toolbar
	bool status=viewMenu->isItemChecked(ID_VIEW_TOOLBAR);
	viewMenu->setItemChecked(ID_VIEW_TOOLBAR, status?FALSE:TRUE);
	break;
      }
    case ID_VIEW_STATUSBAR:
      {
	enableStatusBar(); // toggles statusbar
	bool status=viewMenu->isItemChecked(ID_VIEW_STATUSBAR);
	viewMenu->setItemChecked(ID_VIEW_STATUSBAR, status?FALSE:TRUE);
	break;
      }

      // optionsMenu
    case ID_OPT_ZOOMIN:
      {
	updateZoomMenu(cubebox->zoomIn());
	updateRects();   // fit topwindow to new size 
	break;
      }
    
    case ID_OPT_ZOOMOUT:
      {
	updateZoomMenu(cubebox->zoomOut());
	updateRects();   // fit topwindow to new size
	break;
      }

    case ID_ZOOM_BASE+3:
    case ID_ZOOM_BASE+4:
    case ID_ZOOM_BASE+5:
    case ID_ZOOM_BASE+6:
    case ID_ZOOM_BASE+7:
    case ID_ZOOM_BASE+8:
      {
	cubebox->setZoom(item-ID_ZOOM_BASE);
	updateZoomMenu(item-ID_ZOOM_BASE);
	updateRects();
	break;
      }
   


    case ID_OPT_DEFAULT:
      {
	int cubeDim,cubeSize;
	setDefaultValues(cubeDim,cubeSize);
	if(cubebox)
	  delete cubebox;
	createNewCubebox(cubeDim,cubeSize);
	updatePlayfieldMenu(cubeDim);
	menu->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
	menu->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
	toolbar->setItemEnabled(ID_OPT_ZOOMIN,TRUE);
	toolbar->setItemEnabled(ID_OPT_ZOOMOUT,TRUE);
	emit reset();
	break;
      }

    case ID_OPT_SAVE:
      saveSettings();
      break;

    case ID_COLOR_BASE+1:
    case ID_COLOR_BASE+2:
      changeColor(item-ID_COLOR_BASE);
      break;

    // set new cubedim
	      
    case (ID_FIELD+5):
    case (ID_FIELD+6):
    case (ID_FIELD+7):
    case (ID_FIELD+8):
    case (ID_FIELD+9):
    case (ID_FIELD+10):
       {
	 if(cubebox->dim() != item-ID_FIELD)
	   {
	     int cubeDim=item-ID_FIELD;
	     int zoom=cubebox->zoom();
	     updatePlayfieldMenu(cubeDim);
	     // create new cubebox
	     delete cubebox;
	     createNewCubebox(cubeDim,zoom);
	     emit reset();
	   }
	 break;
      }

      // About
    case ID_HELP_ABOUT:
      {
	QString s;
	s.sprintf(klocale->translate("JumpingCube  version %s \n\n (C) 1998 by Matthias Kiefer \nemail:Matthias.Kiefer@stud.uni-karlsruhe.de"),VERSION);
	KMsgBox::message( 0, klocale->translate("About JumpingCube"), 
		       (const char *)s,
		       KMsgBox::INFORMATION, klocale->translate("OK") );
	break;
      }
   
    case ID_HELP_CONTENTS:
      kapp->invokeHTMLHelp("","");
      break;
      
      

    } // end switch

}

void TopWindow::changePlayer(int newPlayer)
{
  QString s;
  s.sprintf(klocale->translate("on turn: Player %d"),newPlayer);
  statusbar->changeItem(s,ID_STATUS_TURN);
}


void TopWindow::barPositionChanged()
{
  KConfig *config= kapp->getConfig();
  if(config != 0)
    {
      config->writeEntry("MenubarPos",(int)(menu->menuBarPos()));
      config->writeEntry("ToolbarPos",(int)(toolbar->barPos()));
    }
}


void TopWindow::saveProperties(KConfig *config)
{
  // just for the menuentries
  bool status=viewMenu->isItemChecked(ID_VIEW_TOOLBAR);
  config->writeEntry("Toolbar",(int)(status));
  status=viewMenu->isItemChecked(ID_VIEW_STATUSBAR);
  config->writeEntry("Statusbar",(int)(status));
  
  config->writeEntry("Zoom",cubebox->zoom());
  config->writeEntry("CubeDim",cubebox->dim());
  config->writeEntry("Color1",color1->normal().background());
  config->writeEntry("Color2",color2->normal().background());

  cubebox->saveGame(config);

}


void TopWindow::readProperties(KConfig *config)
{
  bool visible=(bool)(config->readNumEntry("Toolbar",(int)(TRUE)));
  viewMenu->setItemChecked(ID_VIEW_TOOLBAR,visible);
  visible=(bool)(config->readNumEntry("Statusbar",(int)(TRUE)));
  viewMenu->setItemChecked(ID_VIEW_STATUSBAR,visible);
  
  int zoom=config->readNumEntry("Zoom",DEFAULT_ZOOM);
  int cubeDim=config->readNumEntry("CubeDim",DEFAULT_DIM);
  color1=new QPalette(config->readColorEntry("Color1",&QColor("darkred")));
  color2=new QPalette(config->readColorEntry("Color2",&QColor("darkblue")));
  

  createNewCubebox(cubeDim,zoom);

  updatePlayfieldMenu(cubeDim);
  updateZoomMenu(cubebox->zoom());

  cubebox->restoreGame(config);
}

#include "topwindow.moc"

