/********************************************************************************** 
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  


#ifndef TOPWINDOW_H
#define TOPWINDOW_H

#include <ktopwidget.h>


#define ID_GAME_QUIT 1
#define ID_GAME_NEW 2
#define ID_GAME_OPEN 3
#define ID_GAME_SAVE 4
#define ID_HELP_ABOUT 11
#define ID_HELP_CONTENTS 12
#define ID_VIEW_TOOLBAR 20
#define ID_VIEW_STATUSBAR 21
#define ID_STATUS_TURN 30
#define ID_OPT_ZOOMIN 41
#define ID_OPT_ZOOMOUT 42
#define ID_OPT_DEFAULT 43
#define ID_OPT_SAVE 44
#define ID_COLOR_BASE 50
#define ID_FIELD 100
#define ID_ZOOM_BASE 150


class KStatusBar;
class KToolBar;
class CubeBox;
class KMenuBar;
class QPalette;


class TopWindow : public KTopLevelWidget
{
  Q_OBJECT
public:
  TopWindow(const char *name=0); // Konstruktor
  ~TopWindow();                  // Destruktor

protected:
  void saveProperties(KConfig *);
  void readProperties(KConfig *);
	
private:
 KToolBar *toolbar;
 KMenuBar *menu;   
 KStatusBar *statusbar;
 QPopupMenu *options,*playfieldMenu;
 QPopupMenu *viewMenu,*zoomMenu;
 QPalette *color1,*color2;
 CubeBox *cubebox;


 void createMenubar();
 void createToolbar();
 void createStatusbar();
 void createNewCubebox(int cubeDim, int cubeSize);
 void updatePlayfieldMenu(int cubeDim);
 void updateZoomMenu(int zoom);
 void setDefaultValues(int &cubeDim, int &cubeSize);
 void changeColor(int player);
 void saveSettings();

	     			
public slots:
  void menuCallback(int);
  void changePlayer(int);
  void barPositionChanged();

signals:
  void reset();


};

#endif // TOPWINDOW_H















